/*
Fallout New Vegas 4GB Helper DLL
Copyright (C) 2010  Renee Stanley (the.wench@wenchy.net)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "stdafx.h"

HANDLE hOutput = 0;
HANDLE hError = 0;

#ifdef _MANAGED
#pragma managed(push, off)
#endif

#define LOTS_OF_NOPS(addr) __asm { \
		__asm mov eax, addr __asm mov ecx, eax \
		__asm mov edx, eax \
		__asm nop __asm nop __asm nop __asm nop  \
		__asm nop __asm nop __asm nop __asm nop  \
		__asm nop __asm nop __asm nop __asm nop  \
		__asm nop __asm nop __asm nop __asm nop  } 

#define REDIRECTABLE_FUNCTION_EX(ret,callconv, name,args) \
	__declspec(noinline) static ret callconv name##_o args { LOTS_OF_NOPS(name##_o) } \
	__declspec(noinline) static ret callconv name##_new args

#define REDIRECT_FUNCTION(module,name,prolog) \
	RedirectFunction((LPBYTE) GetProcAddress(module,#name), (LPBYTE )&name##_new, (LPBYTE )&name##_o, prolog)

int Change4GBValue(LPVOID baseaddress, bool set)
{
	PIMAGE_DOS_HEADER pDOSHeader = static_cast<PIMAGE_DOS_HEADER>( baseaddress );
	if( pDOSHeader->e_magic != IMAGE_DOS_SIGNATURE )
	{ 
		return -1; 
	}

	PIMAGE_NT_HEADERS pNTHeader = reinterpret_cast<PIMAGE_NT_HEADERS>(
	    (PBYTE)baseaddress + pDOSHeader->e_lfanew );

	if(pNTHeader->Signature != IMAGE_NT_SIGNATURE )
	{ 
		return -1; 
	}

	PIMAGE_FILE_HEADER pFileHeader = reinterpret_cast<PIMAGE_FILE_HEADER>( 
		(PBYTE)&pNTHeader->FileHeader );

	PIMAGE_OPTIONAL_HEADER pOptionalHeader = reinterpret_cast<PIMAGE_OPTIONAL_HEADER>(
		(PBYTE)&pNTHeader->OptionalHeader );

	/////////////////////////////////////////////////////////////
	if( IMAGE_NT_OPTIONAL_HDR32_MAGIC != pNTHeader->OptionalHeader.Magic )
	{ 
		return -1; 
	}

	int ret = pFileHeader->Characteristics;

	DWORD old;
	VirtualProtect(&pFileHeader->Characteristics, sizeof(pFileHeader->Characteristics), PAGE_READWRITE, &old);

	if (set)
		pFileHeader->Characteristics |= IMAGE_FILE_LARGE_ADDRESS_AWARE;
	else
		pFileHeader->Characteristics &= ~IMAGE_FILE_LARGE_ADDRESS_AWARE;

	DWORD oldold;
	VirtualProtect(&pFileHeader->Characteristics, sizeof(pFileHeader->Characteristics), old, &oldold);

	return ret;
}

void my_memcpy(LPBYTE dest, CONST BYTE *src, int count)
{
	while (count--) *dest++ = *src++;
}
void my_memcpy(LPVOID dest, LPCVOID src, int count)
{
	my_memcpy((LPBYTE)dest, (CONST BYTE *) src, count);
	
}
void my_memset(LPBYTE dest, BYTE src, int count)
{
	while (count--) *dest++ = src;
}
void my_memset(LPVOID dest, BYTE src, int count)
{
	my_memset((LPBYTE)dest,src,count);
}

void RedirectFunction(LPBYTE old_func, LPBYTE new_func, LPBYTE ret_func, DWORD_PTR bytes)
{
	// This will contain the instructions we need
	BYTE *jmp_start;
	size_t jmp_size;
	DWORD flOldProtect;

	__asm {
		jmp end
start: 
		jmp RedirectFunction
end: 
		mov jmp_start, offset start
		mov jmp_size, offset end
		sub jmp_size, offset start
	}

	// while the old_func is a short jump, resolve it
	while (*old_func == 0xEB) {
		old_func = old_func+2+(signed char)old_func[1];
	}

	if (bytes < jmp_size)
	{
		//printf ("Failed to redirect function at %p to function at %p because the number of bytes available (%i) was smaller than the size required (%i) for the jump instruction\n", old_func, new_func, bytes, jmp_size);
		return;
	}

	if (ret_func)
	{
		// Is a function to jump to another function so go there...
		while (((unsigned char) ret_func[0]) == 0xE9)
			ret_func = ret_func+5 + *(int*)&ret_func[1]; 
		VirtualProtect(ret_func, bytes+jmp_size, PAGE_EXECUTE_READWRITE, &flOldProtect);
		my_memcpy(ret_func,old_func,bytes); 

		if (((unsigned char) ret_func[0]) == 0xE8 || ((unsigned char) ret_func[0]) == 0xE9) // This is a 'call' or 'jmp' instruction, needs fixup
			*(DWORD_PTR*)&ret_func[1] += old_func - ret_func; 
	
		ret_func += bytes;
		my_memcpy(ret_func,jmp_start,jmp_size); 
		ret_func += jmp_size;

		*(size_t*)(ret_func-sizeof(size_t)) = old_func+bytes - ret_func;

	}

	VirtualProtect(old_func, bytes, PAGE_EXECUTE_READWRITE, &flOldProtect);
	my_memset(old_func,0x90,bytes);
	my_memcpy(old_func,jmp_start,jmp_size); old_func += jmp_size;
	*(size_t*)(old_func-sizeof(size_t)) = new_func - old_func;		
}
	
int mystricmp(const char *left, const char *right)
{
	while (*left && *right)
	{
		int l = *left++;
		int r = *right++;

		if (l >= 'a' && l <= 'z') l -= 0x20;
		if (r >= 'a' && r <= 'z') r -= 0x20;

		int res = l - r;
		if (res != 0) return res;
	}

	return *left - *right;
}
int mystrlen(const char *left)
{
	int count = 0;
	while (*left++) count++;
	return count;
}

REDIRECTABLE_FUNCTION_EX(HANDLE,WINAPI,CreateFileA,(LPCSTR lpFileName,DWORD dwDesiredAccess,DWORD dwShareMode,LPSECURITY_ATTRIBUTES lpSecurityAttributes,DWORD dwCreationDisposition,DWORD dwFlagsAndAttributes,HANDLE hTemplateFile))
{
	if (hOutput)
	{
		DWORD numwritten;
		WriteFile(hOutput,"CreateFile: ",12, &numwritten,0);
		WriteFile(hOutput,lpFileName,mystrlen(lpFileName), &numwritten,0);
		WriteFile(hOutput,"\n\r",2, &numwritten,0);
		FlushFileBuffers(hOutput);
	}

	int len = 0;
	CHAR new_filename[MAX_PATH+1];
	while (len < MAX_PATH && (new_filename[len] = lpFileName[len]) != 0) ++len;

	if (len == MAX_PATH) new_filename[MAX_PATH+1] = 0;

	if (len > 12 && !mystricmp(&lpFileName[len-12],"fallout3.exe")) 
	{
		my_memcpy(&new_filename[len-12],"FalloutNV.exe",14);
		lpFileName = new_filename;		
	}
	else if (len > 13 && !mystricmp(&lpFileName[len-13],"falloutnv.4gb"))
	{
		my_memcpy(&new_filename[len-13],"FalloutNV.exe",14);
		lpFileName = new_filename;		
	}

	if (hOutput)
	{
		DWORD numwritten;
		WriteFile(hOutput,"         -> ",12, &numwritten,0);
		WriteFile(hOutput,lpFileName,mystrlen(lpFileName), &numwritten,0);
		WriteFile(hOutput,"\n\r",2, &numwritten,0);
		FlushFileBuffers(hOutput);
	}


	return CreateFileA_o(lpFileName,dwDesiredAccess,dwShareMode,lpSecurityAttributes,dwCreationDisposition,dwFlagsAndAttributes,hTemplateFile);
}

void DwordToString(DWORD num, char string[16])
{
	int count = 0;
	DWORD t = num;

	while (t)
	{
		count++;
		t /= 10;				
	}

	if (count == 0)
	{
		string[0] = '0';
		string[1] = 0;
	}
	else
	{
		string[count] = 0;
		while (num)
		{
			string[--count] = (num%10) + '0';
			num /= 10;				
		}
	}
}
#ifndef USE_THREADED_GETTICKCOUNT
#include <mmsystem.h>
REDIRECTABLE_FUNCTION_EX(DWORD,WINAPI,GetTickCount,())
{
	return timeGetTime();
}

#else
volatile DWORD counter = 0;

REDIRECTABLE_FUNCTION_EX(DWORD,WINAPI,GetTickCount,())
{
	return counter;
}

DWORD CALLBACK TimerThread(LPVOID)
{
	double freq;
	double start;

	SetThreadPriority(GetCurrentThread(),THREAD_PRIORITY_TIME_CRITICAL);

	for (int i = 0; i < 32; i++)
		if (SetThreadAffinityMask(GetCurrentThread(),1<<i) != 0) break;

	LARGE_INTEGER s,f;
	QueryPerformanceCounter(&s);	
	QueryPerformanceFrequency(&f);

	start = s.QuadPart;
	freq = f.QuadPart;
	start -= (GetTickCount()*freq)/1000;

	HMODULE kernel32 = GetModuleHandle(TEXT("KERNEL32"));
	REDIRECT_FUNCTION(kernel32,GetTickCount,5);

	for(;;)
	{
		LARGE_INTEGER c;
		QueryPerformanceCounter(&c);	

		double count = c.QuadPart;
		count -= start;

		counter = (DWORD)((count*1000.0)/freq);

		Sleep(2);
	}
}
#endif

extern "C" BOOL WINAPI _DllMainCRTStartup( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
	if (ul_reason_for_call == DLL_PROCESS_ATTACH)
	{
		HMODULE executable = GetModuleHandle(NULL);

		//TCHAR filename[MAX_PATH];
		//GetModuleFileName(executable,filename,MAX_PATH);
		//MessageBox(0,filename,TEXT("Attach Debugger"),MB_OK);

		//AllocConsole();
		//hOutput = GetStdHandle(STD_OUTPUT_HANDLE);
		//hError = GetStdHandle(STD_ERROR_HANDLE);

		Change4GBValue(LPVOID(executable),false);

		HMODULE kernel32 = GetModuleHandle(TEXT("KERNEL32"));
		
#ifndef USE_THREADED_GETTICKCOUNT
		REDIRECT_FUNCTION(kernel32,GetTickCount,5);
#else
		LARGE_INTEGER f;
		if (QueryPerformanceFrequency(&f))
		{
			DWORD tid;
			HANDLE thread = CreateThread(NULL,0,TimerThread,0,0,&tid);
		}
#endif

		REDIRECT_FUNCTION(kernel32,CreateFileA,5);
		
		LoadLibrary(TEXT("nvse_steam_loader.dll"));
	}
    return TRUE;
}

#ifdef _MANAGED
#pragma managed(pop)
#endif

